import random

# 单个中文姓氏
single_family_name = "赵钱孙李周吴郑王冯陈楮卫蒋沈韩杨朱秦尤许何吕施张孔曹严华金魏陶姜戚谢邹喻柏水窦章云苏潘葛奚范彭郎鲁韦" \
                     "昌马苗凤花方俞任袁柳酆鲍史唐费廉岑薛雷贺倪汤滕殷罗毕郝邬安常乐于时傅皮卞齐康伍余元卜顾孟平黄和穆萧尹" \
                     "姚邵湛汪祁毛禹狄米贝明臧计伏成戴谈宋茅庞熊纪舒屈项祝董梁杜阮蓝闽席季麻强贾路娄危江童颜郭梅盛林刁锺徐" \
                     "丘骆高夏蔡田樊胡凌霍虞万支柯昝管卢莫经房裘缪干解应宗丁宣贲邓郁单杭洪包诸左石崔吉钮龚程嵇邢滑裴陆荣翁" \
                     "荀羊於惠甄麹家封芮羿储靳汲邴糜松井段富巫乌焦巴弓牧隗山谷车侯宓蓬全郗班仰秋仲伊宫宁仇栾暴甘斜厉戎祖武" \
                     "符刘景詹束龙叶幸司韶郜黎蓟薄印宿白怀蒲邰从鄂索咸籍赖卓蔺屠蒙池乔阴郁胥能苍双闻莘党翟谭贡劳逄姬申扶堵" \
                     "冉宰郦雍郤璩桑桂濮牛寿通边扈燕冀郏浦尚农温别庄晏柴瞿阎充慕连茹习宦艾鱼容向古易慎戈廖庾终暨居衡步都耿" \
                     "满弘匡国文寇广禄阙东欧殳沃利蔚越夔隆师巩厍聂晁勾敖融冷訾辛阚那简饶空曾毋沙乜养鞠须丰巢关蒯相查后荆红" \
                     "游竺权逑盖益桓公"

# 单个英文姓氏
en_family_name = ['Laura', 'Melissa', 'Kennet', 'Timothy', 'Betty', 'Scott', 'Mary', 'Susan', 'Brenda', 'Mark', 'Paul',
                  'Donna', 'Richard', 'Thomas', 'Jose', 'Gary', 'Amy', 'Robert', 'Jennifer', 'Steven', 'Charles',
                  'Jessica']
# 英文名
en_name = ['Martin', 'Clark', 'Walker', 'Taylor', 'Lewis', 'Allen', 'Lee', 'Williams', 'Jackson', 'Moore', 'Lopez',
           'Jones', 'Wilson', 'Garcia', 'Harris', 'Davis', 'White', 'Thomas', 'Perez', 'Hall', 'Young', 'Robinson',
           'Smith', 'Gonzalez']
chinese = "一乙二十丁厂七卜人入八九几儿了力乃刀又三于干亏士工土才寸下大丈与万上小口巾山千乞川亿个勺久凡及夕丸么" \
          "广亡门义之尸弓己已子卫也女飞刃习叉马乡丰王井开夫天无元专云扎艺木五支厅不太犬区历尤友匹车巨牙屯比互切" \
          "瓦止少日中冈贝内水见午牛手毛气升长仁什片仆化仇币仍仅斤爪反介父从今凶分乏公仓月氏勿欠风丹匀乌凤勾文六" \
          "方火为斗忆订计户认心尺引丑巴孔队办以允予劝双书幻玉刊示末未击打巧正扑扒功扔去甘世古节本术可丙左厉右石" \
          "布龙平灭轧东卡北占业旧帅归且旦目叶甲申叮电号田由史只央兄叼叫另叨叹四生失禾丘付仗代仙们仪白仔他斥瓜乎" \
          "丛令用甩印乐句匆册犯外处冬鸟务包饥主市立闪兰半汁汇头汉宁穴它讨写让礼训必议讯记永司尼民出辽奶奴加召皮" \
          "边发孕圣对台矛纠母幼丝式刑动扛寺吉扣考托老执巩圾扩扫地扬场耳共芒亚芝朽朴机权过臣再协西压厌在有百存而" \
          "页匠夸夺灰达列死成夹轨邪划迈毕至此贞师尘尖劣光当早吐吓虫曲团同吊吃因吸吗屿帆岁回岂刚则肉网年朱先丢舌" \
          "竹迁乔伟传乒乓休伍伏优伐延件任伤价份华仰仿伙伪自血向似后行舟全会杀合兆企众爷伞创肌朵杂危旬旨负各名多" \
          "争色壮冲冰庄庆亦刘齐交次衣产决充妄闭问闯羊并关米灯州汗污江池汤忙兴宇守宅字安讲军许论农讽设访寻那迅尽" \
          "导异孙阵阳收阶阴防奸如妇好她妈戏羽观欢买红纤级约纪驰巡寿弄麦形进戒吞远违运扶抚坛技坏扰拒找批扯址走抄" \
          "坝贡攻赤折抓扮抢孝均抛投坟抗坑坊抖护壳志扭块声把报却劫芽花芹芬苍芳严芦劳克苏杆杠杜材村杏极李杨求更束" \
          "豆两丽医辰励否还歼来连步坚旱盯呈时吴助县里呆园旷围呀吨足邮男困吵串员听吩吹呜吧吼别岗帐财针钉告我乱利" \
          "秃秀私每兵估体何但伸作伯伶佣低你住位伴身皂佛近彻役返余希坐谷妥含邻岔肝肚肠龟免狂犹角删条卵岛迎饭饮系" \
          "言冻状亩况床库疗应冷这序辛弃冶忘闲间闷判灶灿弟汪沙汽沃泛沟没沈沉怀忧快完宋宏牢究穷灾良证启评补初社识" \
          "诉诊词译君灵即层尿尾迟局改张忌际陆阿陈阻附妙妖妨努忍劲鸡驱纯纱纳纲驳纵纷纸纹纺驴纽奉玩环武青责现表规" \
          "抹拢拔拣担坦押抽拐拖拍者顶拆拥抵拘势抱垃拉拦拌幸招坡披拨择抬其取苦若茂苹苗英范直茄茎茅林枝杯柜析板松" \
          "枪构杰述枕丧或画卧事刺枣雨卖矿码厕奔奇奋态欧垄妻顷转斩轮软到非叔肯齿些虎虏肾贤尚旺具果味昆国昌畅明易" \
          "昂典固忠咐呼鸣咏呢岸岩帖罗帜岭凯败贩购图钓制知垂牧物乖刮秆和季委佳侍供使例版侄侦侧凭侨佩货依的迫质欣" \
          "征往爬彼径所舍金命斧爸采受乳贪念贫肤肺肢肿胀朋股肥服胁周昏鱼兔狐忽狗备饰饱饲变京享店夜庙府底剂郊废净" \
          "盲放刻育闸闹郑券卷单炒炊炕炎炉沫浅法泄河沾泪油泊沿泡注泻泳泥沸波泼泽治怖性怕怜怪学宝宗定宜审宙官空帘" \
          "实试郎诗肩房诚衬衫视话诞询该详建肃录隶居届刷屈弦承孟孤陕降限妹姑姐姓始驾参艰线练组细驶织终驻驼绍经贯" \
          "奏春帮珍玻毒型挂封持项垮挎城挠政赴赵挡挺括拴拾挑指垫挣挤拼挖按挥挪某甚革荐巷带草茧茶荒茫荡荣故胡南药" \
          "标枯柄栋相查柏柳柱柿栏树要咸威歪研砖厘厚砌砍面耐耍牵残殃轻鸦皆背战点临览竖省削尝是盼眨哄显哑冒映星昨" \
          "畏趴胃贵界虹虾蚁思蚂虽品咽骂哗咱响哈咬咳哪炭峡罚贱贴骨钞钟钢钥钩卸缸拜看矩怎牲选适秒香种秋科重复竿段" \
          "便俩贷顺修保促侮俭俗俘信皇泉鬼侵追俊盾待律很须叙剑逃食盆胆胜胞胖脉勉狭狮独狡狱狠贸怨急饶蚀饺饼弯将奖" \
          "哀亭亮度迹庭疮疯疫疤姿亲音帝施闻阀阁差养美姜叛送类迷前首逆总炼炸炮烂剃洁洪洒浇浊洞测洗活派洽染济洋洲" \
          "浑浓津恒恢恰恼恨举觉宣室宫宪突穿窃客冠语扁袄祖神祝误诱说诵垦退既屋昼费陡眉孩除险院娃姥姨姻娇怒架贺盈" \
          "勇怠柔垒绑绒结绕骄绘给络骆绝绞统耕耗艳泰珠班素蚕顽盏匪捞栽捕振载赶起盐捎捏埋捉捆捐损都哲逝捡换挽热恐" \
          "壶挨耻耽恭莲莫荷获晋恶真框桂档桐株桥桃格校核样根索哥速逗栗配翅辱唇夏础破原套逐烈殊顾轿较顿毙致柴桌虑" \
          "监紧党晒眠晓鸭晃晌晕蚊哨哭恩唤啊唉罢峰圆贼贿钱钳钻铁铃铅缺氧特牺造乘敌秤租积秧秩称秘透笔笑笋债借值倚" \
          "倾倒倘俱倡候俯倍倦健臭射躬息徒徐舰舱般航途拿爹爱颂翁脆脂胸胳脏胶脑狸狼逢留皱饿恋桨浆衰高席准座脊症病" \
          "疾疼疲效离唐资凉站剖竞部旁旅畜阅羞瓶兼烤烘烦烧烛烟递涛浙涝酒涉消浩海涂浴浮流润浪浸涨烫涌悟悄悔悦害宽" \
          "家宵宴宾窄容宰案请朗诸读扇袜袖袍被祥课谁调冤谅谈谊剥恳展弱陵陶陷陪娱娘通能难预桑绢绣验继球理捧堵描域" \
          "掩捷排掉堆推掀授教掏掠培接控探据掘职基著勒黄萌萝菌菜萄菊萍菠营械梦梢梅检梳梯副票戚爽聋袭盛雪辅辆虚雀" \
          "堂常匙晨睁眯眼悬野啦晚啄距跃略蛇累唱患唯崖崭崇圈铜铲银甜梨犁移笨笼笛符第敏做袋悠偿偶偷您售停偏假盘船" \
          "斜盒鸽悉欲彩领脚脖脸脱象够猜猪猎猫猛馅馆凑减毫麻痒痕廊康庸鹿盗章竟商族旋望率着盖粘粗粒断剪兽清添淋淹" \
          "渠渐混渔淘液淡深渗情惜惭悼惧惕惊惨惯寇寄宿窑密谋谎祸谜逮敢屠弹随蛋隆隐婚婶颈绩绪续骑绳维绵绸绿琴斑替" \
          "款堪搭塔越趁趋超提堤博揭喜插揪搜煮援搂搅握揉斯期欺联散惹葬葛董葡敬葱落朝辜葵棒棋植森椅椒棵棍棉棚棕惠" \
          "惑逼厨厦硬确雁殖裂雄暂雅辈悲紫辉敞赏掌晴暑最量喷晶喇遇喊跌跑遗蛙蛛蜓喝喂喘喉幅帽赌赔黑铸铺链销锁锄锅" \
          "锈锋锐短智毯鹅剩稍程稀税筐等筑策筛筒答筋筝傲傅牌堡集焦傍储奥街惩御循艇舒番释脾腔鲁猾猴然馋装蛮就痛童" \
          "阔善羡普粪尊道曾焰港湖渣湿温渴滑湾渡游滋溉愤慌惰愧愉慨割寒富窜窝窗遍裕裤裙谢谣谦属屡强粥疏隔隙絮缎缓" \
          "编骗缘瑞魂肆摄摸填搏塌鼓摆携搬摇搞塘摊蒜勤鹊蓝墓幕蓬蓄蒙蒸献禁楚想槐榆楼概赖酬感碍碑碎碰碗碌雷零雾雹" \
          "输督龄鉴睛睡睬鄙盟歇暗照跨跳跪路跟遣蛾蜂嗓置罪罩错锡锣锤锦键锯矮辞稠愁筹签简毁舅鼠催傻像躲微愈遥腰腥" \
          "腹腾腿触解酱痰廉新韵意粮数煎塑慈煤煌源滤滥滔溪溜滚滨粱滩慎誉塞谨福群殿辟障嫌嫁叠缝缠静碧璃墙撇嘉摧截" \
          "誓境摘摔聚蔽慕暮蔑模榴榜榨歌遭酷酿酸磁愿需弊裳颗嗽蜻蜡蝇锹锻舞稳算箩管僚鼻魄貌膜膊膀鲜疑馒裹敲豪膏遮" \
          "腐瘦辣竭端旗精歉熄熔漆漂漫滴演漏慢寨赛察蜜谱嫩翠熊凳骡缩慧撕撒趣趟撑播撞撤增蕉蔬横槽樱橡飘醋醉震霉瞒" \
          "题暴瞎影踢踏踩踪蝶蝴嘱墨镇靠稻黎稿稼箱箭篇僵躺僻德艘膝膛熟摩颜毅糊遵潜潮懂额慰劈操燕薯薪薄颠橘整餐嘴" \
          "蹄器赠默镜赞篮邀衡膨雕磨凝辨辩糖糕燃澡激懒壁避缴戴擦鞠藏霜霞瞧蹈螺穗繁辫赢糟糠燥臂翼骤鞭覆蹦镰翻鹰警" \
          "攀蹲颤瓣爆疆壤耀嚷籍魔灌蠢霸露料益拳粉剧屑桶救得衔婆梁裁搁景践禽腊嫂登愚暖满漠蜘赚聪鞋融醒躁嚼"
cn_punctuation = r"""。？！，、；：“”‘’『』「」（）[]〔〕【】——……·—-～《》〈〉___/"""



import random
import re
import string
import uuid
from datetime import datetime, timedelta
from dateutil.relativedelta import relativedelta
from typing import Union, Optional, List, Tuple

from m_mock.m_random_source import single_family_name, en_family_name, en_name, chinese, cn_punctuation


def in_none(obj: Union[tuple, list, str, None]) -> bool:
    """
    检查对象是否为空或None

    :param obj: 要检查的对象(可以是元组/列表/字符串/None)
    :return: 如果对象为空/None返回True，否则返回False
    """
    if isinstance(obj, (tuple, list)):
        return all(i in ('', None) for i in obj)
    return obj in ('', None)


def tuple_to_str(objects: Union[tuple, list]) -> str:
    """
    将元组/列表转换为字符串

    :param objects: 要转换的元组/列表
    :return: 转换后的字符串
    """
    return ''.join(objects)


def shuffle_string(strings: Union[str, list, tuple]) -> str:
    """
    打乱字符串或序列中的字符

    :param strings: 要打乱的输入(字符串/列表/元组)
    :return: 打乱后的字符串
    """
    if isinstance(strings, str):
        strings = list(strings)
    random.shuffle(strings)
    return ''.join(strings)


class MockPyExpressionException(Exception):
    """
    模拟表达式错误的自定义异常类
    """

    def __init__(self, exception: str = '使用了错误的模拟器表达式', remark: Optional[str] = None):
        super().__init__()
        self.exception = f'{exception}{remark or ""}'

    def __str__(self) -> str:
        return self.exception

    @classmethod
    def min_max_value_exception(cls, min_value: Optional[int], max_value: Optional[int]) -> None:
        """
        验证最小/最大值

        :param min_value: 最小值
        :param max_value: 最大值
        :raises MockPyExpressionException: 如果最小值 >= 最大值
        """
        if min_value is not None and max_value is not None and min_value >= max_value:
            raise cls('最小值不能大于或等于最大值')


_mock_exception = MockPyExpressionException()


class BooleanM:
    """
    布尔值生成器
    """

    @classmethod
    def boolean(cls, min_value: Optional[int] = None, max_value: Optional[int] = None,
                current: Optional[bool] = None) -> bool:
        """
        生成一个布尔值

        :param min_value: 最小值(默认0)
        :param max_value: 最大值(默认1)
        :param current: 当前值(默认True)
        :return: 生成的布尔值
        """
        _mock_exception.min_max_value_exception(min_value, max_value)

        if in_none((max_value, current)) and not in_none(min_value):
            return True

        min_value = 0 if in_none(min_value) else min_value
        max_value = 1 if in_none(max_value) else max_value
        current = True if in_none(current) else current

        return current if random.randint(min_value, max_value) == min_value else not current


m_boolean = BooleanM()


class NaturalM:
    """
    自然数生成器
    """

    @classmethod
    def natural(cls, min_value: Optional[int] = None, max_value: Optional[int] = None) -> int:
        """
        生成一个自然数

        :param min_value: 最小值(默认0)
        :param max_value: 最大值(默认9999999999999999)
        :return: 随机自然数
        """
        min_value = 0 if in_none(min_value) else min_value
        max_value = 9999999999999999 if in_none(max_value) else max_value
        _mock_exception.min_max_value_exception(min_value, max_value)
        return random.randint(min_value, max_value)


m_natural = NaturalM()


class NumberM:
    """
    数字字符串生成器
    """
    number_str_max_length = None

    @classmethod
    def number_str(cls, min_length: Optional[int] = None, max_length: Optional[int] = None) -> str:
        """
        生成随机长度的数字字符串

        :param min_length: 最小长度
        :param max_length: 最大长度
        :return: 随机数字字符串
        """
        _mock_exception.min_max_value_exception(max_length, min_length)
        min_length = random.randint(0, 15) if in_none(min_length) else min_length
        max_length = random.randint(min_length, 16) if in_none(max_length) else max_length

        range_size = cls.number_not_start_with_zero(min_length, max_length)
        return ''.join(str(random.randint(0, 9)) for _ in range(range_size))

    @classmethod
    def number_not_start_with_zero(cls, start_number: int, end_number: int) -> int:
        """
        生成不以零开头的随机数

        :param start_number: 最小值(包含)
        :param end_number: 最大值(包含)
        :return: start_number和end_number之间的随机整数
        """
        start = start_number if start_number != 0 else 1
        end = end_number if end_number != 1 else 2
        return random.randint(start, end)


m_number = NumberM()


class IntegerM:
    """
    整数生成器
    """

    @classmethod
    def integer(cls, min_value: Optional[int] = None, max_value: Optional[int] = None) -> int:
        """
        生成随机整数

        :param min_value: 最小值(默认-9999999999999999)
        :param max_value: 最大值(默认9999999999999999)
        :return: 随机整数
        """
        _mock_exception.min_max_value_exception(min_value, max_value)
        min_value = -9999999999999999 if in_none(min_value) else min_value
        max_value = 9999999999999999 if in_none(max_value) else max_value
        return random.randint(min_value, max_value)


m_integer = IntegerM()


class CharacterM:
    """
    字符生成器
    """

    @classmethod
    def character(cls, character_type: Optional[str] = None) -> str:
        """
        生成随机字符

        :param character_type: 字符类型(小写/大写/数字/符号)
        :return: 随机字符
        """
        if character_type is None:
            character_type = random.choice(('lower', 'upper', 'number', 'symbol'))
        return StringM.string(character_type, 1)


m_character = CharacterM()


class FloatM:
    """
    浮点数生成器
    """

    @classmethod
    def float(cls, min_value: Optional[float] = None, max_value: Optional[float] = None,
              d_min_value: Optional[int] = None, d_max_value: Optional[int] = None) -> float:
        """
        生成随机浮点数

        :param min_value: 整数部分最小值
        :param max_value: 整数部分最大值
        :param d_min_value: 最小小数位数
        :param d_max_value: 最大小数位数
        :return: 随机浮点数
        """

        def __luck() -> bool:
            """75%概率返回True"""
            return random.randint(1, 4) in (1, 2, 3)

        _mock_exception.min_max_value_exception(min_value, max_value)

        min_value = -9999999999999999 if in_none(min_value) else min_value
        max_value = 9999999999999999 if in_none(max_value) else max_value

        if in_none(d_min_value):
            d_min_value = random.randint(2, 5) if __luck() else 0

        if in_none(d_max_value):
            min_d_max_value = d_min_value if 0 < d_min_value < 14 else d_min_value + 1
            d_max_value = random.randint(min_d_max_value + 1, 16)

        decimals = StringM.string_number(d_min_value, d_max_value)

        if __luck():
            while True:
                random_float = random.uniform(min_value, max_value)
                val = str(random_float)
                if '.' in val:
                    break
            int_part = val.split(".")[0]
            int_part = int_part if len(int_part) + len(decimals) <= 15 else int_part[:15 - len(decimals)]
            random_float = float(f'{int_part}.{decimals}')
        else:
            random_float = random.uniform(min_value, max_value)
            if __luck():
                random_float = float(f'{str(random_float)[:-1]}{random.randint(1, 9)}')

        return float(round(random_float, random.randint(d_min_value, d_max_value)))


m_float = FloatM()


class StringM:
    """
    多种类型的字符串生成器
    """

    @classmethod
    def get_random_string_by_source(cls, source: str, min_value: Optional[int] = None,
                                    max_value: Optional[int] = None) -> str:
        """
        从给定源生成随机字符串

        :param source: 字符源
        :param min_value: 最小长度
        :param max_value: 最大长度
        :return: 随机字符串
        """
        _mock_exception.min_max_value_exception(min_value, max_value)

        if in_none(min_value):
            length = random.randint(1, 9)
        elif in_none(max_value):
            length = min_value
        else:
            length = random.randint(min_value, max_value)

        chars = [random.choice(source) for _ in range(length)]
        random.shuffle(chars)
        return ''.join(chars)

    @classmethod
    def string_lower(cls, min_value: Optional[int] = None, max_value: Optional[int] = None) -> str:
        """生成随机小写字符串"""
        return cls.get_random_string_by_source(string.ascii_lowercase, min_value, max_value)

    @classmethod
    def string_upper(cls, min_value: Optional[int] = None, max_value: Optional[int] = None) -> str:
        """生成随机大写字符串"""
        return cls.get_random_string_by_source(string.ascii_uppercase, min_value, max_value)

    @classmethod
    def string_number(cls, min_value: Optional[int] = None, max_value: Optional[int] = None) -> str:
        """生成随机数字字符串"""
        return cls.get_random_string_by_source(string.digits, min_value, max_value)

    @classmethod
    def string_symbol(cls, min_value: Optional[int] = None, max_value: Optional[int] = None) -> str:
        """生成随机符号字符串"""
        return cls.get_random_string_by_source(string.punctuation, min_value, max_value)

    @classmethod
    def strings(cls, min_value: Optional[int] = None, max_value: Optional[int] = None) -> str:
        """生成包含(英文/数字/符号)的随机长度字符串"""
        source = string.ascii_letters + string.digits + string.punctuation
        return cls.get_random_string_by_source(source, min_value, max_value)

    @classmethod
    def chinese(cls, min_value: Optional[int] = None, max_value: Optional[int] = None) -> str:
        """生成随机中文字符串"""
        min_value = 1 if in_none(min_value) else min_value
        return cls.get_random_string_by_source(chinese, min_value, max_value)

    @classmethod
    def english(cls, min_value: Optional[int] = None, max_value: Optional[int] = None) -> str:
        """生成随机英文字符串"""
        min_value = 1 if in_none(min_value) else min_value
        return cls.get_random_string_by_source(string.ascii_letters, min_value, max_value)

    @classmethod
    def cn_symbol(cls, min_value: Optional[int] = None, max_value: Optional[int] = None) -> str:
        """生成随机中文标点符号字符串"""
        return cls.get_random_string_by_source(cn_punctuation, min_value, max_value)

    @classmethod
    def cn_string(cls, min_value: Optional[int] = None, max_value: Optional[int] = None) -> str:
        """生成随机中文字符串(包含中文标点符号)"""

        def __cn_sting() -> str:
            cn_min_value = random.randint(1, max_value - 1)
            cn_str = cls.get_random_string_by_source(chinese, cn_min_value)
            cn_sym = cls.cn_symbol(max_value - cn_min_value)
            combined = list(cn_str + cn_sym)
            random.shuffle(combined)
            return ''.join(combined)

        if min_value is not None and min_value >= 2 and in_none(max_value):
            max_value = min_value
            return __cn_sting()
        elif in_none(max_value) or min_value == 1:
            return cls.get_random_string_by_source(cn_punctuation + chinese, min_value, max_value)
        return __cn_sting()

    @classmethod
    def string(cls, *args) -> Optional[str]:
        """
        根据类型生成随机字符串

        :param args: 类型和长度参数
        :return: 随机字符串或None(如果无效)
        """
        if len(args) <= 1 or isinstance(args[0], int):
            return cls.strings(*args)
        elif 2 <= len(args) <= 3:
            string_type = args[0]
            new_args = args[1:]

            generators = {
                'lower': cls.string_lower,
                'upper': cls.string_upper,
                'number': cls.string_number,
                'symbol': cls.string_symbol,
                'string': cls.strings,
                'chinese': cls.chinese,
                'english': cls.english,
                'cn_symbol': cls.cn_symbol,
                'cn_string': cls.cn_string
            }

            if string_type in generators:
                return generators[string_type](*new_args)
            return cls.get_random_string_by_source(string_type, *new_args)
        elif len(args) > 3:
            raise MockPyExpressionException('最多只允许3个参数')
        return None


m_string = StringM()


class DateM:
    """
    日期和时间生成器
    """

    @classmethod
    def datetime_calculate(cls, date_time: datetime, time_interval: str, format_str: Optional[str]) -> Optional[str]:
        """
        基于时间间隔计算日期时间

        :param date_time: 基准日期时间
        :param time_interval: 时间间隔(例如'+1min')
        :param format_str: 输出格式
        :return: 格式化后的日期时间字符串或None
        """
        calculate = time_interval[:1]
        unit = re.search('[a-zA-Z]+', time_interval).group(0)
        amount = int(time_interval[1:-len(unit)])

        if 'hours'.startswith(unit):
            delta = timedelta(hours=amount)
        elif 'minutes'.startswith(unit):
            delta = timedelta(minutes=amount)
        elif 'milliseconds'.startswith(unit):
            delta = timedelta(milliseconds=amount)
        elif 'microseconds'.startswith(unit):
            delta = timedelta(microseconds=amount)
        elif 'seconds'.startswith(unit):
            delta = timedelta(seconds=amount)
        elif 'days'.startswith(unit):
            delta = timedelta(days=amount)
        elif 'month'.startswith(unit):
            return (date_time + relativedelta(months=amount)).strftime(format_str)
        elif 'week'.startswith(unit):
            return (date_time + relativedelta(weeks=amount)).strftime(format_str)
        else:
            return None

        if calculate == '+':
            return (date_time + delta).strftime(format_str)
        elif calculate == '-':
            return (date_time - delta).strftime(format_str)
        return None

    @classmethod
    def datetime(cls, format_str: Optional[str] = None, time_interval: Optional[str] = None) -> str:
        """
        生成格式化日期时间字符串

        :param format_str: 格式字符串(默认'%Y-%m-%d %H:%M:%S')
        :param time_interval: 时间间隔(例如'+1min')
        :return: 格式化后的日期时间字符串
        :raises MockPyExpressionException: 无效的时间间隔
        """
        if time_interval and (len(time_interval) <= 1 or time_interval[:1] not in ('+', '-')):
            raise MockPyExpressionException(remark="正确的时间表达式应为'+1h'或'-1h'")

        curr_time = datetime.now()
        if time_interval:
            return cls.datetime_calculate(curr_time, time_interval, format_str or '%Y-%m-%d %H:%M:%S')
        return curr_time.strftime(format_str or '%Y-%m-%d %H:%M:%S')

    @classmethod
    def date(cls, format_str: Optional[str] = None, time_interval: Optional[str] = None) -> str:
        """
        生成格式化日期字符串

        :param format_str: 格式字符串(默认'%Y-%m-%d')
        :param time_interval: 时间间隔(例如'-1d')
        :return: 格式化后的日期字符串
        """
        return cls.datetime(format_str or '%Y-%m-%d', time_interval)

    @classmethod
    def timestamp(cls) -> int:
        """生成当前时间戳(毫秒)"""
        return int(datetime.now().timestamp() * 1000)

    @classmethod
    def time(cls, format_str: Optional[str] = None, time_interval: Optional[str] = None) -> str:
        """
        生成格式化时间字符串

        :param format_str: 格式字符串(默认'%H:%M:%S')
        :param time_interval: 时间间隔(例如'-1h')
        :return: 格式化后的时间字符串
        """
        return cls.datetime(format_str or '%H:%M:%S', time_interval)

    @staticmethod
    def get_current_week(date: Optional[str] = None, format_str: str = '%Y-%m-%d') -> Tuple[str, str]:
        """
        获取当前周的周一和周日日期

        :param date: 日期字符串(可选)
        :param format_str: 格式字符串
        :return: (周一, 周日)日期元组
        """
        duty_date = datetime.strptime(date, format_str) if date else datetime.today()
        monday = sunday = duty_date
        one_day = timedelta(days=1)

        while monday.weekday() != 0:
            monday -= one_day
        while sunday.weekday() != 6:
            sunday += one_day

        return monday.strftime(format_str), sunday.strftime(format_str)

    @classmethod
    def now(cls, unit: Optional[str] = None, format_str: str = '%Y-%m-%d %H:%M:%S') -> Optional[str]:
        """
        获取指定单位的当前日期时间

        :param unit: 时间单位(年/月/周/日/时/分/秒)
        :param format_str: 格式字符串
        :return: 格式化后的日期时间字符串或None
        """
        default_format = '%Y-%m-%d %H:%M:%S'
        now = cls.date(format_str='%Y') + '-01-01 00:00:00'

        unit_map = {
            'year': lambda: datetime.strptime(now, default_format).strftime(format_str),
            'month': lambda: datetime.strptime(cls.date(format_str='%Y-%m') + '-01 00:00:00',
                                               default_format).strftime(format_str),
            'week': lambda: datetime.strptime(cls.get_current_week()[1] + ' 00:00:00',
                                              default_format).strftime(format_str),
            'day': lambda: datetime.strptime(cls.date(format_str='%Y-%m-%d') + ' 00:00:00',
                                             default_format).strftime(format_str),
            'hour': lambda: datetime.strptime(cls.date(format_str='%Y-%m-%d  %H') + ':00:00',
                                              default_format).strftime(format_str),
            'minute': lambda: datetime.strptime(cls.date(format_str='%Y-%m-%d  %H:%M') + ':00',
                                                default_format).strftime(format_str),
            'second': lambda: cls.datetime(format_str=format_str),
            None: lambda: cls.datetime(format_str=format_str)
        }

        return unit_map.get(unit, lambda: None)()


m_date = DateM()


class HelperM:
    """
    辅助函数
    """

    @classmethod
    def pick(cls, pick_list: Union[str, list, tuple]) -> Union[str, int, float]:
        """
        从列表中随机选择一个元素

        :param pick_list: 要选择的列表
        :return: 随机元素
        :raises MockPyExpressionException: 空输入
        """
        if in_none(pick_list):
            raise MockPyExpressionException('pick_list不能为空')
        if isinstance(pick_list, str):
            pick_list = eval(pick_list)
        return random.choice(pick_list)


m_helper = HelperM()


class NameM:
    """
    姓名生成器
    """

    @classmethod
    def first(cls) -> str:
        """获取随机英文姓氏"""
        return random.choice(en_family_name)

    @classmethod
    def last(cls) -> str:
        """获取随机英文名字"""
        return random.choice(en_name)

    @classmethod
    def name(cls, middle: Optional[Union[str, bool, int]] = None) -> str:
        """
        生成英文姓名

        :param middle: 是否包含中间名
        :return: 英文姓名字符串
        """
        if middle and str(middle).lower() in ('true', '1'):
            first = cls.first()
            last = cls.last()
            for _ in range(20):  # 尝试找到不同的中间名
                mid = cls.first()
                if mid != first:
                    return f'{first} {mid} {last}'
            return f'{first} {last} {last}'  # 如果找不到不同的中间名则回退
        return f'{cls.first()} {cls.last()}'

    @classmethod
    def cfirst(cls, length: Optional[int] = None) -> str:
        """
        生成随机中文名字

        :param length: 名字长度(1-3)
        :return: 中文名字字符串
        :raises MockPyExpressionException: 无效的长度
        """
        length = random.randint(1, 3) if length is None else length
        if not 1 <= length <= 3:
            raise MockPyExpressionException('无效的名字长度')
        return m_string.chinese(length, length + 1)[:length]

    @classmethod
    def clast(cls) -> str:
        """获取随机中文姓氏"""
        return random.choice(single_family_name)

    @classmethod
    def cname(cls, length: Optional[int] = None) -> str:
        """
        生成中文全名

        :param length: 名字长度(2-3)
        :return: 中文全名
        :raises MockPyExpressionException: 无效的长度
        """
        length = random.randint(2, 3) if length is None else length
        if not 2 <= length <= 3:
            raise MockPyExpressionException('无效的名字长度')
        return cls.clast() + cls.cfirst(length - 1)


m_name = NameM()


class MiscellaneousM:
    """
    杂项生成器
    """
    increment_start = 0

    @classmethod
    def uuid(cls) -> str:
        """生成随机UUID"""
        return str(uuid.uuid4())

    @classmethod
    def id(cls) -> str:
        """生成随机中国身份证号"""
        return cls.generate_random_id()

    @classmethod
    def increment(cls, step: Optional[int] = None) -> str:
        """
        自增计数器

        :param step: 增量步长(默认1)
        :return: 自增后的值(字符串形式)
        """
        step = 1 if step is None else step
        cls.increment_start += step
        return f"{cls.increment_start}"

    @staticmethod
    def get_random_date(start_year: int = 1960, end_year: int = 2020) -> str:
        """生成随机出生日期"""
        start = datetime(start_year, 1, 1).date()
        end = datetime(end_year, 12, 31).date()
        days_between = (end - start).days
        random_days = random.randrange(days_between)
        return (start + timedelta(days=random_days)).strftime("%Y%m%d")

    @staticmethod
    def calculate_checksum(id17: str) -> str:
        """计算身份证校验码"""
        weights = [7, 9, 10, 5, 8, 4, 2, 1, 6, 3, 7, 9, 10, 5, 8, 4, 2]
        checksums = '10X98765432'
        sum_product = sum(int(id17[i]) * weights[i] for i in range(17))
        return checksums[sum_product % 11]

    @classmethod
    def generate_random_id(cls) -> str:
        """生成随机中国身份证号码"""
        region_code = str(random.randint(100000, 699999))  # 随机地区码
        birth_date = cls.get_random_date()  # 随机出生日期
        sequence_code = str(random.randint(0, 999)).zfill(3)  # 随机顺序码
        id17 = region_code + birth_date + sequence_code  # 组成前17位
        checksum = cls.calculate_checksum(id17)  # 计算校验码
        return f"{id17}{checksum}"  # 完整身份证号


m_miscellaneous = MiscellaneousM()


class TextM:
    """
    文本生成器
    """

    @classmethod
    def paragraph(cls, min_sentences: int = 3, max_sentences: int = 10) -> str:
        """
        生成英文段落

        :param min_sentences: 最小句子数
        :param max_sentences: 最大句子数
        :return: 英文段落
        """
        num = random.randint(min_sentences, max_sentences)
        pick_list = [',', '!', '?']
        sentences = [f'{m_string.english(1, 10)[:-1]}{random.choice(pick_list)}' for _ in range(num)]
        return f"{''.join(sentences)[:-1]}."

    @classmethod
    def cparagraph(cls, min_sentences: int = 3, max_sentences: int = 10) -> str:
        """
        生成中文段落

        :param min_sentences: 最小句子数
        :param max_sentences: 最大句子数
        :return: 中文段落
        """
        num = random.randint(min_sentences, max_sentences)
        pick_list = ['，', '！', '？']
        sentences = [f'{m_string.chinese(1, 10)[:-1]}{random.choice(pick_list)}' for _ in range(num)]
        return f"{''.join(sentences)[:-1]}。"

    @classmethod
    def sentence(cls, min_words: int = 3, max_words: int = 10) -> str:
        """
        生成英文句子

        :param min_words: 最小单词数
        :param max_words: 最大单词数
        :return: 英文句子
        """
        return f"{m_string.english(min_words, max_words)}."

    @classmethod
    def csentence(cls, min_words: int = 3, max_words: int = 10) -> str:
        """
        生成中文句子

        :param min_words: 最小词数
        :param max_words: 最大词数
        :return: 中文句子
        """
        return f"{m_string.chinese(min_words, max_words)}。"

    @classmethod
    def mix_sentence(cls, min_words: int = 1, max_words: int = 10) -> str:
        """
        生成混合语言句子

        :param min_words: 最小词数
        :param max_words: 最大词数
        :return: 混合语言句子
        """
        parts = [
            cls.sentence(min_words, max_words),
            cls.csentence(min_words, max_words),
            m_string.string_number(min_words, max_words)
        ]
        return shuffle_string(''.join(parts))


m_text = TextM()



import re
from m_mock import m_random


class MockM:
    """
    模拟数据生成器核心类

    通过关键字映射调用对应的随机数据生成方法
    """

    def __init__(self):
        # 创建关键字映射字典，将关键字关联到对应的模块和方法
        self.keyword_mapping = {
            # 日期时间相关
            'date': (m_random.m_date, 'date'),
            'datetime': (m_random.m_date, 'datetime'),
            'time': (m_random.m_date, 'time'),
            'timestamp': (m_random.m_date, 'timestamp'),
            'now': (m_random.m_date, 'now'),

            # 数字相关
            'float': (m_random.m_float, 'float'),
            'natural': (m_random.m_natural, 'natural'),
            'integer': (m_random.m_integer, 'integer'),
            'boolean': (m_random.m_boolean, 'boolean'),

            # 字符串相关
            'character': (m_random.m_character, 'character'),
            'string': (m_random.m_string, 'string'),

            # 辅助功能
            'pick': (m_random.m_helper, 'pick'),
            'increment': (m_random.m_miscellaneous, 'increment'),

            # 中文姓名相关
            'cfirst': (m_random.m_name, 'cfirst'),
            'clast': (m_random.m_name, 'clast'),
            'cname': (m_random.m_name, 'cname'),

            # 英文姓名相关
            'first': (m_random.m_name, 'first'),
            'last': (m_random.m_name, 'last'),
            'name': (m_random.m_name, 'name'),

            # 标识符相关
            'id': (m_random.m_miscellaneous, 'id'),
            'uuid': (m_random.m_miscellaneous, 'uuid'),

            # 文本相关
            'mix_sentence': (m_random.m_text, 'mix_sentence'),
            'sentence': (m_random.m_text, 'sentence'),
            'csentence': (m_random.m_text, 'csentence'),
            'paragraph': (m_random.m_text, 'paragraph'),
            'cparagraph': (m_random.m_text, 'cparagraph'),
        }

    def mock(self, mock_str: str) -> any:
        """
        根据输入字符串生成模拟数据

        :param mock_str: 模拟字符串表达式，如"@date()"、"@name()"
        :return: 生成的模拟数据
        :raises MockPyExpressionException: 当表达式格式错误时抛出
        """
        # 1. 提取关键字
        keyword = self.get_mocker_key(mock_str)

        # 2. 提取参数
        args = self.get_mocker_params_to_tuple(mock_str)

        # 3. 通过关键字获取对应的模块和方法
        module_method_pair = self.keyword_mapping.get(keyword)

        if module_method_pair:
            module, method_name = module_method_pair
            try:
                # 4. 动态调用对应方法并传入参数
                return getattr(module, method_name)(*args)
            except Exception as e:
                raise m_random.MockPyExpressionException(f"调用方法失败: {method_name}, 错误: {str(e)}")
        else:
            # 5. 如果没有匹配的关键字，返回原始字符串
            return mock_str

    @classmethod
    def get_mocker_key(cls, mock_str: str) -> str:
        """
        从模拟字符串中提取关键字

        :param mock_str: 模拟字符串，如"@date()"、"@name"
        :return: 提取的关键字，如"date"、"name"
        :raises MockPyExpressionException: 当字符串不以@开头时抛出
        """
        if not mock_str.startswith('@'):
            raise m_random.MockPyExpressionException("模拟表达式必须以@开头")

        # 如果不是以括号结尾，直接返回@后的内容
        if not mock_str.endswith(')'):
            return mock_str[1:]

        # 使用正则提取@和(之间的内容
        regular = r'(?<=(@)).*?(?=\()'
        match = re.search(regular, mock_str)
        if not match:
            raise m_random.MockPyExpressionException("无法提取关键字")
        return match.group(0)

    @classmethod
    def get_mocker_params_to_tuple(cls, mock_params: str) -> tuple:
        """
        将模拟参数转换为元组形式

        :param mock_params: 参数字符串，如"('%Y.%m.%d %H:%M:%S','+1')"
        :return: 参数元组，如('%Y.%m.%d %H:%M:%S', '+1')
        """
        # 如果没有括号，返回空元组
        if '(' not in mock_params or not mock_params.endswith(')'):
            return ()

        # 提取括号内的内容
        regular = r'[\\(（].*[\\)）]$'
        match = re.search(regular, mock_params)
        if not match:
            return ()

        group = match.group(0)

        # 处理空参数情况
        if group == '()':
            return ()

        # 处理单参数情况，确保eval能正确解析
        if not group.endswith(','):
            group = f'{group[:-1]},)'

        try:
            # 安全地将字符串转换为元组
            args = eval(group)
            return args if isinstance(args, tuple) else (args,)
        except:
            return ()


# 创建全局实例
m_mock = MockM()